package de.tu_darmstadt.tk.SmartHomeNetworkSim.core;

import java.util.Collection;

/**
 * Connection between two or more SmartDevices, which sends packages according
 * to a specified protocol
 * 
 * @author Andreas T. Meyer-Berg
 */
public interface Connection {

	/**
	 * Returns the Link, which the connection uses
	 * 
	 * @return
	 */
	public Link getLink();

	/**
	 * Returns the SmartDevice which started this connection
	 * 
	 * @return source SmartDevice
	 */
	public SmartDevice getSource();

	/**
	 * Returns the SmartDevices that are Part of this Connection
	 * 
	 * @return SmartDevices participating in this Connection
	 */
	public Collection<SmartDevice> getParticipants();

	/**
	 * Removes the SmartDevice from the Connection. Should create terminating
	 * packages, that are returned on the next call of simulateTimeIntervall.
	 * If the Connection will be fully closed, source should be set to null and
	 * getTerminationPackages should return the lastPackages. If the connection still
	 * continues, a new Device should become the source.
	 * The Calling Method should remove the Connection in the SmartDevice and
	 * add the Connection to Model.terminatedConnections(), if source was set to null.
	 * 
	 * @param sd SmartDevice to Remove
	 */
	public void removeSmartDevice(SmartDevice sd);

	/**
	 * Simulate Time Step
	 * 
	 * @param startTime
	 * @param duration
	 * @return packets that were sent
	 */
	public Collection<Packet> simulateTimeIntervall(long startTime, long duration);
	
	/**
	 * Returns the Packets, created to terminate the connection
	 * 
	 * @param startTime
	 * @return packets that were sent
	 */
	public Collection<Packet> getTerminationPackages(long startTime);

}
